#!/usr/bin/env python
from __future__ import print_function

import numpy as np
import unsio

#
# class CSsnapshot
#
class CUNS_OUT:
    """Output Operations on UNS snapshots

    This class provide methods for saving data in an UNS compatible snapshots

    """
    __uns=None
    __verbose=False
    __verbose_debug=False
    __float32=True
    __loaded=False
    __outfile=None
    __uns_type=None
    select=None
    __is_valid=False
    __data_type=None

# -----------------------------------------------------
# constructor
#
    def __init__(self,outfile, uns_type, float32=True, verbose=False, verbose_debug=False):
        """
        The Constructor for CUNS_OUT class

        example :
           - instantiate an object to save file "mysnap" in gadget3 with float precision

           import unsio.output as uns_out
           my_out=uns_out.CUNS_OUT("mysnap","gadget3")

        Args:
            outfile (str) : output file name

            uns_type(str) : UNS file format
                            available format for saving are :
                            - gadget2, gadget3(hdf5), nemo

            float32 (bool): save data in float value or double value.

                            example:
                              float32=True (save in float)
                              float32=False (save in double)
                            (default: True)
        """
        self.__float32 = float32
        self.__outfile = outfile
        self.__uns_type = uns_type
        self.__verbose = verbose
        self.__verbose_debug = verbose_debug
        if self.__float32:
            if self.__verbose_debug:
                print("saving in SINGLE precision floating values")
            self.__data_type = np.float32
            self.__uns = unsio.CunsOut(self.__outfile,self.__uns_type)
        else:
            if self.__verbose_debug:
                print("saving in DOUBLE precision floating values")
            self.__data_type = np.float64
            self.__uns = unsio.CunsOutD(self.__outfile,self.__uns_type)
        self.__is_valid = True
# -----------------------------------------------------
# setData
#
    def setData(self,data_array,select,tag=None):
        """
        General method to set Data Array/Value respecting to components to an uns snapshot

        example:
           - set numpy array pos as positions for gas component
             status=obj.setData(pos,"gas","pos")

           - set float value timex as simulation time
             status=obj.setData(timex,"time")

        Args:
           data_array (array|float) : 1D numpy_array or single value

           select (str) : component gas,stars,disk,halo,bndry or bulge

           tag    (str) : array to set
                          example :
                              tag="pos"
                              tag="acc"
                              pos,vel,mass,rho...etc see https://projets.lam.fr/projects/unsio/wiki/GetDataDescription

        IMPORTANT : if 'tag' is None, then 'select' is treated as 'tag' (see above)

        Return :
            status : 1 if success, 0 otherwise
        """
        if not self.__is_valid:
            raise RuntimeError("CUNS_OUT is not valid")

        ok=0
        if tag is None: # only one value
            tab_value_F=["time"]
            tab_value_I=["nbody","nsel"]
            if tab_value_I.count(select) > 0:
                ok=self.__uns.setValueI(select,data_array)
            else:
                ok=self.__uns.setValueF(select,data_array)
        else:
            if tag != "id": #not id, then we save float arrays
                ok = self.__uns.setArrayF(select,tag,data_array)
            else: # save id's array of integers
                ok = self.__uns.setArrayI(select,tag,data_array)

        if self.__verbose_debug:
            print("setData : [select=%s, tag=%s, ok=%d ]"%(select,tag,ok))

        return ok

# -----------------------------------------------------
# save
#
    def save(self):
        """save data presumbly set on file"""
        if not self.__is_valid:
            raise RuntimeError("CUNS_OUT is not valid")

        self.__uns.save()

# -----------------------------------------------------
# close
#
    def close(self):
        """close open file"""
        if not self.__is_valid:
            raise RuntimeError("CUNS_OUT is not valid")

        self.__uns.close()



#!/usr/bin/env python
from __future__ import print_function

#from uns_simu import *
import os,sys
import numpy as np
import tempfile

import unsio
from unsio.input import *
from unsio.output import *
from .check_platform import *

import subprocess
#
# class CTestunsio
#
class CTestunsio:
    """
    This class aims to test UNSIO library
    """

    __3D       = [3,"pos","vel","acc"]
    __1D       = [1,"mass","pot"]
    __1Dgas    = [1,"rho","hsml","metal","temp","nh","sfr"]
    __1Dstars  = [1,"metal","age"]
    __1Dextra  = [1,"XX","yXz","ZZ"]
    __1Dint    = [1,"id"]

    __comp = {'halo':[__1D,__3D],'gas':[__1D,__3D,__1Dgas],'stars':[__1D,__3D,__1Dstars],
              'disk':[__1D,__3D],'bndry':[__1D,__3D],'bulge':[__1D,__3D],
              'EXTRA':[__1Dextra]}
    __compNemo = { 'all':[__1D,__3D,__1Dgas] }

    __fdout  = None
    __out    = None
    __single = None
# -----------------------------------------------------
#
    def __init__(self,nbody=None,seed=666,single=True,verbose=None, uns2uns=False, out=None):
        self.__nbody   = nbody
        self.__verbose = verbose
        self.__seed    = seed
        self.__single  = single
        self.__uns2uns = uns2uns

        self.__out = out
        if out is not None:
            try:
                self.__fdout = open(out,"w")
            except:
                print("Impossible to write into file file[%s]\n"%(out))
                sys.exit()
        # print banner
        print("\n\n-----------------------------------------------------------",file=self.__fdout)
        print("Testing UNSIO library",file=self.__fdout)
        print("library version [%s]"%(unsio.getVersion()),file=self.__fdout)
        print("python-unsio version [%s]"%(unsio.__version__),file=self.__fdout)
        print("-----------------------------------------------------------",file=self.__fdout)

        # detect plateform
        info=CPlateform(fdout=self.__fdout)
        info.display()

        self.__initSeed(self.__seed)

# -----------------------------------------------------
#
    def __initSeed(self,seed=None):
        if seed is None:
            seed=self.__seed

        np.random.seed(seed)

# -----------------------------------------------------
#
    def __dataF(self,n):
        if (self.__single):
            x=np.float32(np.random.sample(n))
        else:
            x=np.float64(np.random.sample(n))
        return x
# -----------------------------------------------------
#
    def __saveArray(self,comp,attr,dim,real=True):

        if real:
            data=self.__dataF(self.__nbody*dim)
            # ok=self.__unso.setArrayF(comp,attr,data) # save real arrays
            ok = self.__unso.setData(data,comp,attr)
        else:
            # ok=self.__unso.setArrayI(comp,attr,np.arange(self.__nbody*dim,dtype=np.int32)) # save real arrays    
            if comp != "EXTRA":
                ok = self.__unso.setData(np.arange(self.__nbody*dim,dtype=np.int32),comp,attr)
            else:
                ok = 1
        return ok
# -----------------------------------------------------
#
    def saveModel(self,filename=None,unstype="gadget3",single=True):
        """
        save model in requested format
        """
        self.__initSeed() # reset random generator

        # create temporary file
        if filename is None:
            f = tempfile.NamedTemporaryFile()
            self.__model_file = f.name
            f.close()
        else:
            self.__model_file = filename

        fff = self.__model_file

        if (single):
            print("SINGLE precision floating values",file=self.__fdout)
        else:
            print("DOUBLE precision floating values",file=self.__fdout)
        ## SAVE FILE
        # instantiate output object
        self.__unso=CUNS_OUT(self.__model_file,unstype, float32=single, verbose_debug=self.__verbose)
        print("\nSaving in ",unstype," format......",file=self.__fdout)
        # self.__unso.setValueF("time",0)      # save time
        self.__unso.setData(0,"time")
        select_comp =  self.__comp # comp for gadget2 gadget3
        if unstype=="nemo":
            select_comp =  self.__compNemo # comp for nemo

        #for comp,all_array in select_comp.iteritems():
        for comp,all_array in select_comp.items():
            print ("[%-6s] : "%(comp),file=self.__fdout,end="")
            # save reals array
            for block_array in all_array:
                dim=block_array[0]
                for array in block_array[1:]:
                    ok=self.__saveArray(comp,array,dim,real=True)  # save real
                    print(" %s/%d"%(array,ok),file=self.__fdout, end="")


            # save integer arrays
            dim=self.__1Dint[0]
            for array in self.__1Dint[1:]:
                ok=self.__saveArray(comp,array,dim,real=False) # save integer
                print(" %s/%d"%(array,ok),file=self.__fdout, end="")

            print("\n",file=self.__fdout)

        self.__unso.save() # trigger save ops
        self.__unso.close()

        if self.__uns2uns : # test uns2uns
            ff = tempfile.NamedTemporaryFile()
            myfile = ff.name+"_"+unstype
            ff.close()

            if single:
                outfloat="float=t"
            else:
                outfloat="float=f"
            cmd="uns2uns in=%s out=%s select=%s type=%s %s"%(self.__model_file,myfile,"all",unstype,outfloat)
            if self.__verbose:
                print("<%s>"%(cmd),file=self.__fdout)
            #subprocess.call([cmd],shell=True)
            subprocess.call(["uns2uns","in="+self.__model_file,"out="+myfile,"select=all","type="+unstype,outfloat])

            # sys.exit()
            os.remove(self.__model_file)
            self.__model_file=myfile

        if self.__verbose:
            print("Outfile = [%s]"%(self.__model_file),file=self.__fdout)

# -----------------------------------------------------
#
    def __compareArray(self,comp,attr,dim,real=True):

        if real:# float
            data_ref=self.__dataF(self.__nbody*dim)
        else: #integer
            data_ref=np.arange(self.__nbody,dtype=np.int32)

        if (comp=="EXTRA") :
            ok,data=self.__unsi.getData("STREAM",attr)
        else :
            ok,data=self.__unsi.getData(comp,attr)
        #print("%f "%(data_ref[0]),end="",file=sys.stderr)
        if ok:
            #print ("Checking comp[%s] attribute [%s] size [%d] "%(comp,attr,data.size),type(data),data,file=sys.stderr)
            #print (" <%s>"%(attr),file=sys.stderr,end="")
            ok=(data_ref==data).all()
            if not ok:
                print("\n[ERROR !!!] Inconsitency:  <%s> [%s]"%(comp,attr),data_ref.size,data.size,file=self.__fdout)
                print(data_ref,data,file=self.__fdout)
                sys.exit()
        return ok
# -----------------------------------------------------
#
    def __compareModel(self,unstype="gadget3",single=True):
        """
        load model from disk and compare with generated values
        """
        self.__initSeed() # reset random generator

        if not os.path.isfile(self.__model_file):
            print("File [%s] does not exist, aborting..\n"%(self.__model_file),file=self.__fdout)
            sys.exit()

        # instantiate CSnapshot object
        self.__unsi=CUNS_IN(self.__model_file,float32=single)
        self.__unsi.nextFrame() # load snaphot

        select_comp =  self.__comp # comp for gadget2 gadget3
        if unstype=="nemo":
            select_comp =  self.__compNemo # comp for nemo

        for comp,all_array in select_comp.items():
            print ("checking [%-6s] : "%(comp),file=self.__fdout,end="")
            # compare real array
            for block_array in all_array:
                dim=block_array[0]
                for array in block_array[1:]:
                    ok=self.__compareArray(comp,array,dim,real=True)
                    if comp == "EXTRA" and unstype=="gadget2":
                        assert ok == 1
                    print(" %s/%d"%(array,ok),file=self.__fdout, end="")

            # compare integer arrays
            dim=self.__1Dint[0]
            for array in self.__1Dint[1:]:
                ok=self.__compareArray(comp,array,dim,real=False)
                if comp != "EXTRA":
                        assert ok == 1
                print(" %s/%d"%(array,ok),file=self.__fdout, end="")

            print("\n",file=self.__fdout)

        self.__unsi.close()

# -----------------------------------------------------
#
    def testIO(self,model=["gadget3","gadget2","nemo"]):
        """
        test models snasphot that unsio knows howto dump on a file
        """

        
        for mm in model:
            print ("Testing model [%s]"%(mm),file=self.__fdout)
            self.saveModel(unstype=mm,single=self.__single)
            print ("\n\nComparing model [%s]"%(mm),file=self.__fdout)
            self.__compareModel(unstype=mm,single=self.__single)
            # remove temporary file
            if not self.__uns2uns: # remove only if not using uns2uns
                os.remove(self.__model_file)
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# commandLine, parse the command line
def commandLine():
    import argparse
     # help
    parser = argparse.ArgumentParser(description="Test UNSIO library",
                                    formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    # options
    parser.add_argument('--nbody', help='#bodies to test', type=int, default=100000)
    parser.add_argument('--out', help='save result in a file', type=str, default=None)
    parser.add_argument('--verbose',help='verbose mode',dest="verbose", action="store_true", default=False)
    parser.add_argument('--double',help='test with double real',dest="double", action="store_true", default=False)
    parser.add_argument('--uns2uns',help='save intermediate file with uns2uns',dest="uns2uns", action="store_true", default=False)
     # parse
    args = parser.parse_args()

    # start main funciton
    process(args)

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# commandLine, parse the command line
def process(args):
    uns=CTestunsio(nbody=args.nbody,single=not args.double, uns2uns=args.uns2uns, out=args.out, verbose=args.verbose)
    #uns.saveModel("")
    uns.testIO()


# -----------------------------------------------------
# main program
if __name__ == '__main__':
  commandLine()



#!/usr/bin/env python
from __future__ import print_function

import numpy as np
import unsio

#
# class CSsnapshot
#
class CUNS_IN:
    """Input Operations on UNS snapshots

    This class provide methods for loading data stored in an UNS compatible snapshots

    """
    __uns=None
    __verbose=False
    __float32=True
    __loaded=False
    simname=None
    select=None
#
# constructor
#
    def __init__(self,simname,select="all",times="all",float32=True,verbose=False,verbose_debug=False):
        """
        The Constructor for CUNS_IN class

        example :
           - instantiate an object to load components gas and stars of snapshot "mysnap"
             with double precision

           import unsio.input as uns_in
           my_in=uns_in.CUNS_IN("mysnap","gas,stars",float32=False)

        Args:
            simname (str) : snapshot's file name or simulation's name

            select  (str) : selected components to be loaded. It's a list of components
                            separated with coma ",".
                            Known components are: disk,gas,halo,bndry,bulge,all

                            example :
                              select="gas,disk"
                              select="halo"
                            (default: all)

            times   (str) : specify on which range of time data must be loaded.

                            example :
                              times="100"  (load time 100)
                              times="0:10" (load 0<=time<=10)
                              times="all"  (load all times)
                            (default: all)

            float32 (bool): load data in float value or double value.

                            example:
                              float32=True (load in float)
                              float32=False (load in double)
                            (default: True)
        """
        #select=select.encode('ascii')
        #times=times.encode('ascii')
        self.__verbose=verbose
        self.simname=simname
        self.select=select

        self.__vdebug=verbose_debug
        if self.__vdebug:
            print(">> 32 bits floating value",float32)
        if simname is not None:
            #self.simname=self.simname.encode('ascii')
            if float32:
                if self.__vdebug:
                    print("32 bits",self.simname,self.select,times,verbose,type(self.simname),type(self.select))
                self.__uns=unsio.CunsIn(self.simname,self.select,times,verbose)
            else:
                if self.__vdebug:
                    print("64 bits",simname,select,times,verbose)
                self.__uns=unsio.CunsInD(self.simname,self.select,times,verbose)
            if not self.__uns.isValid():
                raise RuntimeError("UNS not valid")
        else:
            None

    def debugOn(self):
        self.__vdebug=True
#
# nextFrame
#
    def nextFrame(self,bits=""):
        """
        Load the next snapshot. Data retreival depend from bits (array selected) and
        components selected during instanciation.

        Args :
             bits (str) : It's a based string variable which control which arrays will be loaded according to components selected
                          Each characters of the string represents a different arrays.

                          example :
                            bits="mxv" will load masses (m), positions(x) and velocities(v)
                            bits="xRH" will load postions(x), densities(R) and hsml(H)

                          for a complete bits explanation, please visit :
                          https://projets.lam.fr/projects/unsio/wiki/UnsioVariablesUtilisation#How-to-select-data-at-loading
                          (default: all)

        Return :
            int: 1 success (something has been loaded), 0 otherwise

        """
        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        else:
            ok=self.__uns.nextFrame(bits)
            if ok:
                self.__loaded=True
            return ok
#
# close
#
    def close(self):
        """close opened snapshot"""

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        self.__uns.close()
#
# kill
#
    def kill(self):
        """close opened snapshot"""

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        self.close()
#
# getInterfaceType
#
    def getInterfaceType(self):
        """return uns snapshot's Interface type"""

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        return self.__uns.getInterfaceType()
#
# getFileStructure
#
    def getFileStructure(self):
        """return uns snapshot's File structure"""

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        return self.__uns.getFileStructure()
#
# getFileName
#
    def getFileName(self):
        """return uns snapshot's File name"""

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        return self.__uns.getFileName()
#
# getData
#
    def getData(self,select,tag=None,data_type=np.float32):
        """
        General method to get Data Array/Value respecting to components
        example :
           - to get positions from gas component
             status,pos=obj.getData("gas","pos")

           - to get concatened positions from gas and stars component
             status,pos=obj.getData("gas,stars","pos")

           - to get simulation Time
             status,time=obj.getData("time")

        Args:
            select (str) : component or list of components separeted with a coma
                           example :
                              select="gas"
                              select="gas,stars"
            tag    (str) : array to get
                           example :
                              tag="pos"
                              tag="acc"
                              pos,vel,mass,rho...etc see https://projets.lam.fr/projects/unsio/wiki/GetDataDescription

            IMPORTANT : if 'tag' is None, then 'select' is treated as 'tag' (see above)

        Return :
            status,numpy_array       (if tag is not None)
            status,value             (if tag is None)
            in both case, status=1 if success, 0 otherwise
        """

        if not self.__uns.isValid():
            raise RuntimeError("UNS not valid")
        if not self.__loaded:
            raise RuntimeError("Snapshot not loaded")

        status=1
        if tag != "id":
            ret_data=np.zeros(0,dtype=data_type) # initalyse an empty array
        else:
            ret_data=np.zeros(0,dtype=np.int32) # initalyse an empty array

        for comp in select.split(","):
            if self.__vdebug:
                print ("comp :", comp)
            ok,type,data=self.__getData(comp,tag,data_type)
            if (ok) :
                if type==2: # array
                    ret_data = np.append(ret_data,data)
                else:
                    ret_data = data
            else:
                status=0 # one component missing
            if self.__vdebug and type==2:
                print ("ok=",ok," data=",data.size," ret_data=",ret_data.size)
        return status,ret_data
#
# __getData
#
    def __getData(self,comp_value,tag=None,data_type=np.float32):
        if self.__vdebug:
            print ("array=",tag,data_type)
        if tag is None: # only one value
            tab_value_F=["time"]
            tab_value_I=["nbody","nsel","ngas","nstars","ndisk","nhalo","nbulge","nbndry","nvarh"]
            ok=0
            ret_data=np.zeros(1,dtype=data_type)
            data=0
            # proceed __uns.getValue()
            if tab_value_I.count(comp_value) > 0:
                ok,ret_data[0]=self.__uns.getValueI(comp_value)
                data=int(ret_data[0])
            else:
                ok,ret_data[0]=self.__uns.getValueF(comp_value)
                data=float(ret_data[0])
            #ret_data[0]=data
            if self.__vdebug:
                print ("ok,data",ok,data)
            return ok,1,data
        else:
            if tag != "id":
                ok,data=self.__uns.getArrayF(comp_value,tag)
            else:
                ok,data=self.__uns.getArrayI(comp_value,tag)
            return ok,2,data




from __future__ import print_function

from unsio.py_unsio import *
try:
	from unsio.py_unsio import *
except:
	import sys
	print("Unable to import [unsio.py_unsio]....",file=sys.stderr)

from .version import version as __version__

#from . import test, csnapshot

#from .csnapshot import *



#!/usr/bin/env python
from __future__ import print_function

import platform
import sys

class CPlateform:
    """
    Detect running platform
    """
    __fdout=None # file descriptor
    __out=None   # file name
    __is_open=False # out file is not open
    
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # 
    def __init__(self,out=None, fdout=None):
        """
        Set file descriptor to save platform information

        """
        self.__fdout = fdout
        self.__out   = out
        
        if (fdout is None and out is not None): # open a new file
            try:
                self.__fdout=open(out,"w")
                self.__is_open=True
            except:
                print("Impossible to write into file file[%s]\n"%(out))
                sys.exit()
        else :
            if (fdout is None and out is None) : # sys.stderr defaul
                self.__fdout=sys.stderr # default file desc if fdout is None
                
                
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # display
    
    def display(self):
        """
        Display platform information
        """
        try:
            myplateform=str(platform.dist())
        except:
            myplateform="none"
        print("""
Python version    : %s
dist              : %s
linux_distribution: %s
system            : %s
machine           : %s
platform          : %s
version           : %s
mac_ver           : %s
uname             : %s
        """ % (
        sys.version.split('\n'),
        myplateform,
        self.__linux_distribution(),
        platform.system(),
        platform.machine(),
        platform.platform(),
        platform.version(),
        platform.mac_ver(),
        platform.uname(),
        ),
        file=self.__fdout)
        if self.__is_open:
            self.__fdout.close()
            
    # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
    # __linux_distribution

    def __linux_distribution(self):
        try:
            return platform.linux_distribution()
        except:
            return "N/A"

# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# commandLine, parse the command line
def commandLine():
    import argparse
    # help
    parser = argparse.ArgumentParser(description="Print info on platform",
                                     formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    # options
    parser.add_argument('--out', help='save info in outfilename, if None on screen', type=str, default=None)

    args=parser.parse_args()
    process(args)
    
# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# commandLine, parse the command line
def process(args):
    info=CPlateform(args.out)
    info.display()
    

# -----------------------------------------------------
# main program
if __name__ == '__main__':
  commandLine()




#!/usr/bin/env python
#
# In the example below, we load a RAMSES simulation that
# we save in gadget3 file format
#

import unsio.input as uns_in  # unsio reading module

#myfile="/home/jcl/output_00004" # input RAMSES simulation
myfile="/home/jcl/Documents/EAGLE_SIMULATION/snapshot_028_z000p000/snap_028_z000p000.0.hdf5" # input RAMSES simulation
# we instantiate object
my_in=uns_in.CUNS_IN(myfile,"gas,stars") # We select components GAS and STARS
#
# Reading
#
if my_in.nextFrame(): # load snapshot
  # read stars positions
  status,poss=my_in.getData("stars","pos")
  print("stars",status,poss,poss.size/3)

  # read gas positions
  print("gas")
  status,posg=my_in.getData("gas","pos")
  print("stars",status,posg)
  # read gas densities
  status,rho=my_in.getData("gas","rho")
  print("rho",status,rho)
  # read time simulation
  status,timex=my_in.getData("time")
#
# Writing
#
import unsio.output as uns_out # unsio writing module

myoutfile="snapshot.g3" # output file name
# we instantiate object
my_out=uns_out.CUNS_OUT(myoutfile,"gadget3") # select gadget3 output format

# prepare data to be saved
# set time
status=my_out.setData(timex,"time")
# set positions for stars
status=my_out.setData(poss,"stars","pos")
# set positions for gas
status=my_out.setData(posg,"gas","pos")
# set density for gas
status=my_out.setData(rho,"gas","rho")

# write on file system
my_out.save()
# close
my_out.close()



#!/usr/bin/env python
# ============================================================================
# Copyright Jean-Charles LAMBERT - 2008-2022
#           Centre de donneeS Astrophysiques de Marseille (CeSAM)       
# e-mail:   Jean-Charles.Lambert@lam.fr                                      
# address:  Aix Marseille Universite, CNRS, LAM 
#           Laboratoire d'Astrophysique de Marseille                          
#           Pole de l'Etoile, site de Chateau-Gombert                         
#           38, rue Frederic Joliot-Curie                                     
#           13388 Marseille cedex 13 France                                   
#           CNRS UMR 7326    
# ============================================================================

"""
UNSIO : Universal Nbody Snapshot Input Output

"""

import numpy
import os
import sys
from setuptools import setup, Extension
from setuptools.command.build_py import build_py as _build_py
import platform

#  find out numpy include directory.
try:
    numpy_include = numpy.get_include()
except AttributeError:
    numpy_include = numpy.get_numpy_include()

if sys.version_info[:2] < (2, 7) or (3, 0) <= sys.version_info[:2] < (3, 4):
    raise RuntimeError("Python version 2.7 or >= 3.4 required.")

# detect python version (2 or 3)
pyversion = "3"
if sys.version_info[0] < 3:
    pyversion = ""

with open('README.md', 'r') as f:
    long_description = f.read()

# trick to add SWIG generated module
# see https://stackoverflow.com/questions/12491328/python-distutils-not-include-the-swig-generated-module
# and especially : https://stackoverflow.com/questions/29477298/setup-py-run-build-ext-before-anything-else/48942866#48942866
# and the fix for python2 https://stackoverflow.com/questions/1713038/super-fails-with-error-typeerror-argument-1-must-be-type-not-classobj-when/1713052#1713052

class build_py(_build_py, object):
    def run(self):
        self.run_command("build_ext")
        if pyversion == "":  # python2
            return super(build_py, self).run()
        else:                # python3
            return super().run()


#
# version management
#
MAJOR = '1'
MINOR = '0'
MICRO = '1'
VERSION = '%s.%s.%s' % (MAJOR, MINOR, MICRO)

#
# write_version : write unsio version in py/unsio/version.py file
# it's imported from __init__.py as __version__


def write_version_py(filename='py/unsio/version.py'):
    cnt = """
# THIS FILE IS GENERATED FROM PYTHON-UNSIO SETUP.PY
#
version = '%(version)s'
"""
    a = open(filename, 'w')
    try:
        a.write(cnt % {'version': VERSION})

    finally:
        a.close()

#
# setup_package :
#

def setup_package():
    # generate version
    write_version_py()

    # detect if linux
    machine=platform.platform()
    LINUX=machine.lower().find("linux")
    if LINUX >= 0:
        # linux detected
        EXTRALINKARGS=[]
    else:
        # should be a macosx machine, mandatory for py36
        EXTRALINKARGS=['-rpath', '/usr/lib']

    # metada for setup
    metadata = dict(
        # name='python'+pyversion+'-unsio',
        name='python-unsio',
        version=VERSION,
        description='A python wrapper to unsio',
        long_description=long_description,
        long_description_content_type='text/markdown',
        author='Jean-Charles LAMBERT',
        author_email='jean-charles.lambert@lam.fr',
        url='https://projets.lam.fr/projects/unsio/wiki',
        license='CeCILL2.1 (https://opensource.org/licenses/CECILL-2.1)',
        classifiers=[
            "Intended Audience :: Developers",
            "Intended Audience :: Science/Research",
            "Programming Language :: C",
            "Programming Language :: C++",
            "Programming Language :: Python",
            "Programming Language :: Python :: 2.7",
            "Programming Language :: Python :: 3.4",
            "Programming Language :: Python :: 3.5",
            "Programming Language :: Python :: 3.6",
            "Programming Language :: Python :: 3.7",
            "Programming Language :: Python :: 3.8",
            "Programming Language :: Python :: 3.9",
            "Programming Language :: Python :: 3.10",
            "Programming Language :: Python :: 3.11",
            "Topic :: Scientific/Engineering :: Astronomy",
            "Topic :: Software Development"],
        platforms=["Linux", "Mac OS-X", "Unix"],
        python_requires='>=2.7,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*',
        package_dir={'': 'py'},  # all packages are under 'py' directory
        packages=['unsio', 'unsio/test'],
        # cmdclass = {'install':Build_ext_first},
        # cmdclass={'build': CustomBuild, 'install': CustomInstall},
        cmdclass={'build_py': build_py},
        py_modules=['unsio/py_unsio'],
        ext_modules=[
            Extension("unsio/_py_unsio",
                      sources=["py/unsio/py_unsio.i"],
                      swig_opts=['-c++', '-modern', '-Isrc',
                                 '-I./py/unsio', '-Iswig'],
                      include_dirs=[numpy_include, 'src'],
                      libraries=['unsio'],
                      library_dirs=[os.environ['HOME']+'/local/unsio/lib',
                                    os.environ['HOME']+'/local/unsio/lib/x86_64-linux-gnu',
                                               '/usr/lib64', '/lib64'],
                      runtime_library_dirs=[
                                            os.environ['HOME']+'/local/unsio/lib',
											os.environ['HOME']+'/local/unsio/lib/x86_64-linux-gnu',
                                            '/usr/lib64', '/lib64'],
                       extra_link_args=EXTRALINKARGS
                      )
        ],
        entry_points={
            "console_scripts": [
                "test_unsio_lib = unsio.test.ctestunsio:commandLine",
            ],
        },
        install_requires=['numpy'],
        setup_requires=['numpy']
        #
    )
    setup(**metadata)


#
# main
#
if __name__ == '__main__':
    setup_package()
